@@ -28,8 +28,7 @@
 static int32_t smpShiftValue;
 static uint32_t oldAudioFreq, tickTimeLenInt, randSeed = INITIAL_DITHER_SEED;
 static uint64_t tickTimeLenFrac;
-static float fSqrtPanningTable[256+1];
-static double dAudioNormalizeMul, dPrngStateL, dPrngStateR;
+static float fSqrtPanningTable[256+1], fAudioNormalizeMul, fPrngStateL, fPrngStateR;
 static voice_t voice[MAX_CHANNELS * 2];
 
 // globalized
@@ -103,11 +102,11 @@ void setAudioAmp(int16_t amp, int16_t masterVol, bool bitDepth32Flag)
 	amp = CLAMP(amp, 1, 32);
 	masterVol = CLAMP(masterVol, 0, 256);
 
-	double dAmp = (amp * masterVol) / (32.0 * 256.0);
+	float fAmp = (amp * masterVol) / (32.0f * 256.0f);
 	if (!bitDepth32Flag)
-		dAmp *= 32768.0;
+		fAmp *= 32768.0f;
 
-	dAudioNormalizeMul = dAmp;
+	fAudioNormalizeMul = fAmp;
 }
 
 void decreaseMasterVol(void)
@@ -194,17 +193,15 @@ void audioSetInterpolationType(uint8_t interpolationType)
 	// set sinc LUT pointers
 	if (config.interpolation == INTERPOLATION_SINC8)
 	{
-		fSinc_1 = fSinc8_1;
-		fSinc_2 = fSinc8_2;
-		fSinc_3 = fSinc8_3;
+		for (int32_t i = 0; i < SINC_KERNELS; i++)
+			fSinc[i] = fSinc8[i];
 
 		audio.sincInterpolation = true;
 	}
 	else if (config.interpolation == INTERPOLATION_SINC16)
 	{
-		fSinc_1 = fSinc16_1;
-		fSinc_2 = fSinc16_2;
-		fSinc_3 = fSinc16_3;
+		for (int32_t i = 0; i < SINC_KERNELS; i++)
+			fSinc[i] = fSinc16[i];
 
 		audio.sincInterpolation = true;
 	}
@@ -237,11 +234,10 @@ static void voiceUpdateVolumes(int32_t i, uint8_t status)
 
 	// now we need to handle volume ramping
 
-	const bool voiceSampleTrigger = !!(status & IS_Trigger);
-
-	if (voiceSampleTrigger)
+	const bool voiceTriggerFlag = !!(status & CS_TRIGGER_VOICE);
+	if (voiceTriggerFlag)
 	{
-		// sample is about to start, ramp out/in at the same time
+		// voice is about to start, ramp out/in at the same time
 
 		if (v->fCurrVolumeL > 0.0f || v->fCurrVolumeR > 0.0f)
 		{
@@ -265,7 +261,7 @@ static void voiceUpdateVolumes(int32_t i, uint8_t status)
 		v->fCurrVolumeL = v->fCurrVolumeR = 0.0f;
 	}
 
-	if (!voiceSampleTrigger && v->fTargetVolumeL == v->fCurrVolumeL && v->fTargetVolumeR == v->fCurrVolumeR)
+	if (!voiceTriggerFlag && v->fTargetVolumeL == v->fCurrVolumeL && v->fTargetVolumeR == v->fCurrVolumeR)
 	{
 		v->volumeRampLength = 0; // no ramp needed for now
 	}
@@ -275,7 +271,7 @@ static void voiceUpdateVolumes(int32_t i, uint8_t status)
 		const float fVolumeRDiff = v->fTargetVolumeR - v->fCurrVolumeR;
 
 		float fRampLengthMul;
-		if (status & IS_QuickVol) // duration of 5ms
+		if (status & CS_USE_QUICK_VOLRAMP) // 5ms duration
 		{
 			v->volumeRampLength = audio.quickVolRampSamples;
 			fRampLengthMul = audio.fQuickVolRampSamplesMul;
@@ -368,37 +364,38 @@ void updateVoices(void)
 
 		ch->status = 0;
 
-		if (status & IS_Vol)
+		// for "render individual tracks" mode during WAV render
+		if (ch->dontRenderThisChannel)
+			continue;
+
+		if (status & CS_UPDATE_VOL)
 		{
 			v->fVolume = ch->fFinalVol; // 0.0f .. 1.0f
 			v->scopeVolume = (uint8_t)((ch->fFinalVol * (SCOPE_HEIGHT*4.0f)) + 0.5f);
 		}
 
-		if (status & IS_Pan)
+		if (status & CS_UPDATE_PAN)
 			v->panning = ch->finalPan;
 
-		if (status & (IS_Vol + IS_Pan))
+		if (status & (CS_UPDATE_VOL + CS_UPDATE_PAN)) // for vol and/or pan updates
 			voiceUpdateVolumes(i, status);
 
-		if (status & IS_Period)
+		if (status & CF_UPDATE_PERIOD)
 		{
-			const double dVoiceHz = dPeriod2Hz(ch->finalPeriod);
+			v->delta = period2VoiceDelta(ch->finalPeriod);
 
-			// set voice delta
-			v->delta = (int64_t)((dVoiceHz * audio.dHz2MixDeltaMul) + 0.5); // Hz -> fixed-point delta (rounded)
 			if (audio.sincInterpolation)
 			{
-				// decide which sinc LUT to use according to the resampling ratio
 				if (v->delta <= sincRatio1)
-					v->fSincLUT = fSinc_1;
+					v->fSincLUT = fSinc[0];
 				else if (v->delta <= sincRatio2)
-					v->fSincLUT = fSinc_2;
+					v->fSincLUT = fSinc[1];
 				else
-					v->fSincLUT = fSinc_3;
+					v->fSincLUT = fSinc[2];
 			}
 		}
 
-		if (status & IS_Trigger)
+		if (status & CS_TRIGGER_VOICE)
 			voiceTrigger(i, ch->smpPtr, ch->smpStartPos);
 	}
 }
@@ -406,7 +403,7 @@ void updateVoices(void)
 void resetAudioDither(void)
 {
 	randSeed = INITIAL_DITHER_SEED;
-	dPrngStateL = dPrngStateR = 0.0;
+	fPrngStateL = fPrngStateR = 0.0f;
 }
 
 static inline int32_t random32(void)
@@ -421,26 +418,26 @@ static inline int32_t random32(void)
 static void sendSamples16BitStereo(void *stream, uint32_t sampleBlockLength)
 {
 	int32_t out32;
-	double dOut, dPrng;
+	float fOut, fPrng;
 
 	int16_t *streamPtr16 = (int16_t *)stream;
 	for (uint32_t i = 0; i < sampleBlockLength; i++)
 	{
 		// left channel - 1-bit triangular dithering
-		dPrng = random32() * (1.0 / (UINT32_MAX+1.0)); // -0.5 .. 0.5
-		dOut = (double)audio.fMixBufferL[i] * dAudioNormalizeMul;
-		dOut = (dOut + dPrng) - dPrngStateL;
-		dPrngStateL = dPrng;
-		out32 = (int32_t)dOut;
+		fPrng = (float)random32() * (1.0f / (UINT32_MAX+1.0f)); // -0.5f .. 0.5f
+		fOut = audio.fMixBufferL[i] * fAudioNormalizeMul;
+		fOut = (fOut + fPrng) - fPrngStateL;
+		fPrngStateL = fPrng;
+		out32 = (int32_t)fOut;
 		CLAMP16(out32);
 		*streamPtr16++ = (int16_t)out32;
 
 		// right channel - 1-bit triangular dithering
-		dPrng = random32() * (1.0 / (UINT32_MAX+1.0)); // -0.5 .. 0.5
-		dOut = (double)audio.fMixBufferR[i] * dAudioNormalizeMul;
-		dOut = (dOut + dPrng) - dPrngStateR;
-		dPrngStateR = dPrng;
-		out32 = (int32_t)dOut;
+		fPrng = (float)random32() * (1.0f / (UINT32_MAX+1.0f)); // -0.5f .. 0.5f
+		fOut = audio.fMixBufferR[i] * fAudioNormalizeMul;
+		fOut = (fOut + fPrng) - fPrngStateR;
+		fPrngStateR = fPrng;
+		out32 = (int32_t)fOut;
 		CLAMP16(out32);
 		*streamPtr16++ = (int16_t)out32;
 
@@ -451,20 +448,20 @@ static void sendSamples16BitStereo(void *stream, uint32_t sampleBlockLength)
 
 static void sendSamples32BitFloatStereo(void *stream, uint32_t sampleBlockLength)
 {
-	double dOut;
+	float fOut;
 
 	float *fStreamPtr32 = (float *)stream;
 	for (uint32_t i = 0; i < sampleBlockLength; i++)
 	{
 		// left channel
-		dOut = (double)audio.fMixBufferL[i] * dAudioNormalizeMul;
-		dOut = CLAMP(dOut, -1.0, 1.0);
-		*fStreamPtr32++ = (float)dOut;
+		fOut = audio.fMixBufferL[i] * fAudioNormalizeMul;
+		fOut = CLAMP(fOut, -1.0f, 1.0f);
+		*fStreamPtr32++ = fOut;
 
 		// right channel
-		dOut = (double)audio.fMixBufferR[i] * dAudioNormalizeMul;
-		dOut = CLAMP(dOut, -1.0, 1.0);
-		*fStreamPtr32++ = (float)dOut;
+		fOut = audio.fMixBufferR[i] * fAudioNormalizeMul;
+		fOut = CLAMP(fOut, -1.0f, 1.0f);
+		*fStreamPtr32++ = fOut;
 
 		// clear what we read from the mixing buffer
 		audio.fMixBufferL[i] = audio.fMixBufferR[i] = 0.0f;
@@ -806,7 +803,7 @@ static void fillVisualsSyncBuffer(void)
 		c->smpStartPos = s->smpStartPos;
 
 		c->pianoNoteNum = 255; // no piano key
-		if (songPlaying && ui.instEditorShown && (c->status & IS_Period) && !s->keyOff)
+		if (songPlaying && ui.instEditorShown && (c->status & CF_UPDATE_PERIOD) && !s->keyOff)
 		{
 			const int32_t note = getPianoKey(s->finalPeriod, s->finetune, s->relativeNote);
 			if (note >= 0 && note <= 95)
