@@ -93,7 +93,8 @@ enum // CHECKBOXES
 	CB_CONF_STRETCH_IMAGE,
 	CB_CONF_PIXEL_FILTER,
 
-	CB_WAV_BPM_MODE,
+	// WAV RENDERER
+	CB_WAV_RENDER_INDIVIDUAL_TRACKS,
 
 	NUM_CHECKBOXES
 };
