@@ -20,6 +20,7 @@
 #include "ft2_midi.h"
 #include "ft2_bmp.h"
 #include "ft2_structs.h"
+#include "ft2_keyboard.h"
 
 #define NUM_CURSORS 6
 
@@ -30,6 +31,21 @@ static int16_t mouseShape;
 static int32_t mouseModeGfxOffs, mouseBusyGfxFrame;
 static SDL_Cursor *cursors[NUM_CURSORS];
 
+#if defined __APPLE__ && defined __aarch64__
+void armMacGhostMouseCursorFix(void)
+{
+	/* M E G A K L U D G E:
+	** The mouse cursor can sometimes change back to OS stock
+	** (or show both stock and custom mouse) on Macs with a notch
+	** (ARM based) in fullscreen mode. Weird, right?!
+	**
+	** XXX: Can this cause stuttering or performance issues?
+	*/
+	if (video.fullscreen)
+		SDL_SetCursor(NULL); // forces redraw
+}
+#endif
+
 static bool setSystemCursor(SDL_Cursor *cur)
 {
 	if (config.specialFlags2 & USE_OS_MOUSE_POINTER)
@@ -77,7 +93,7 @@ bool createMouseCursors(void) // creates scaled SDL surfaces for current mouse p
 
 	for (int32_t i = 0; i < NUM_CURSORS; i++)
 	{
-		int32_t scaleFactor = video.yScale;
+		int32_t scaleFactor = video.mouseCursorUpscaleFactor;
 		if (scaleFactor > 8) // just in case
 			scaleFactor = 8;
 
@@ -157,8 +173,8 @@ bool createMouseCursors(void) // creates scaled SDL surfaces for current mouse p
 
 		if (i == 3) // text edit cursor bias
 		{
-			hotX = 2 * video.xScale;
-			hotY = 6 * video.yScale;
+			hotX = 2 * video.mouseCursorUpscaleFactor;
+			hotY = 6 * video.mouseCursorUpscaleFactor;
 		}
 
 		cursors[i] = SDL_CreateColorCursor(surface, hotX, hotY);
@@ -443,7 +459,7 @@ void mouseWheelHandler(bool directionUp)
 	if (ui.sysReqShown || editor.editTextFlag)
 		return;
 
-	if (ui.extended)
+	if (ui.extendedPatternEditor)
 	{
 		if (mouse.y <= 52)
 		{
@@ -540,7 +556,12 @@ void mouseWheelHandler(bool directionUp)
 		if (ui.sampleEditorShown)
 		{
 			if (mouse.y >= 174 && mouse.y <= 328)
-				directionUp ? mouseZoomSampleDataIn() : mouseZoomSampleDataOut();
+			{
+				if (keyb.leftShiftPressed)
+					directionUp ? scrollSampleDataLeft() : scrollSampleDataRight();
+				else
+					directionUp ? mouseZoomSampleDataIn() : mouseZoomSampleDataOut();
+			}
 		}
 		else if (ui.patternEditorShown)
 		{
@@ -564,7 +585,7 @@ static bool testPatternDataMouseDown(void)
 {
 	if (ui.patternEditorShown)
 	{
-		const int32_t y1 = ui.extended ? 56 : 176;
+		const int32_t y1 = ui.extendedPatternEditor ? 71 : 176;
 		const int32_t y2 = ui.pattChanScrollShown ? 382 : 396;
 
 		if (mouse.y >= y1 && mouse.y <= y2 && mouse.x >= 29 && mouse.x <= 602)
@@ -815,8 +836,8 @@ void handleLastGUIObjectDown(void)
 
 void updateMouseScaling(void)
 {
-	if (video.renderW > 0.0) video.fMouseXMul = (float)SCREEN_W / video.renderW;
-	if (video.renderH > 0.0) video.fMouseYMul = (float)SCREEN_H / video.renderH;
+	if (video.renderW > 0) video.dMouseXMul = (double)SCREEN_W / video.renderW;
+	if (video.renderH > 0) video.dMouseYMul = (double)SCREEN_H / video.renderH;
 }
 
 void readMouseXY(void)
@@ -832,7 +853,14 @@ void readMouseXY(void)
 		return;
 	}
 
-	if (video.useDesktopMouseCoords)
+	if (video.fullscreen)
+	{
+		mouse.buttonState = SDL_GetMouseState(&mx, &my);
+
+		mouse.absX = mx;
+		mouse.absY = my;
+	}
+	else
 	{
 		mouse.buttonState = SDL_GetGlobalMouseState(&mx, &my);
 
@@ -845,63 +873,55 @@ void readMouseXY(void)
 		mx -= windowX;
 		my -= windowY;
 	}
-	else
-	{
-		// special mode for KMSDRM (XXX: Confirm that this still works...)
-		mouse.buttonState = SDL_GetMouseState(&mx, &my);
-
-		mouse.absX = mx;
-		mouse.absY = my;
-	}
 
 	mouse.rawX = mx;
 	mouse.rawY = my;
 
 	if (video.fullscreen)
 	{
-		// centered fullscreen mode (not stretched) needs further coord translation
-		if (!(config.specialFlags2 & STRETCH_IMAGE))
+		// if software mouse is enabled, warp mouse inside render space
+		if (!(config.specialFlags2 & HARDWARE_MOUSE))
 		{
-			// if software mouse is enabled, warp mouse inside render space
-			if (!(config.specialFlags2 & HARDWARE_MOUSE))
+			bool warpMouse = false;
+
+			if (mx < video.renderX)
 			{
-				bool warpMouse = false;
-
-				if (mx < video.renderX)
-				{
-					mx = video.renderX;
-					warpMouse = true;
-				}
-				else if (mx >= video.renderX+video.renderW)
-				{
-					mx = (video.renderX + video.renderW) - 1;
-					warpMouse = true;
-				}
-
-				if (my < video.renderY)
-				{
-					my = video.renderY;
-					warpMouse = true;
-				}
-				else if (my >= video.renderY+video.renderH)
-				{
-					my = (video.renderY + video.renderH) - 1;
-					warpMouse = true;
-				}
-
-				if (warpMouse)
-					SDL_WarpMouseInWindow(video.window, mx, my);
+				mx = video.renderX;
+				warpMouse = true;
+			}
+			else if (mx >= video.renderX+video.renderW)
+			{
+				mx = (video.renderX + video.renderW) - 1;
+				warpMouse = true;
 			}
 
-			// convert fullscreen coords to window (centered image) coords
-			mx -= video.renderX;
-			my -= video.renderY;
+			if (my < video.renderY)
+			{
+				my = video.renderY;
+				warpMouse = true;
+			}
+			else if (my >= video.renderY+video.renderH)
+			{
+				my = (video.renderY + video.renderH) - 1;
+				warpMouse = true;
+			}
+
+			if (warpMouse)
+				SDL_WarpMouseInWindow(video.window, mx, my);
 		}
+
+		// convert fullscreen coords to window (centered image) coords
+		mx -= video.renderX;
+		my -= video.renderY;
 	}
 
-	// multiply coords by video upscaling factors (don't round)
-	mouse.x = (int32_t)(mx * video.fMouseXMul);
-	mouse.y = (int32_t)(my * video.fMouseYMul);
+	// kludge: this can happen and prevent buttons from pressing on the very first row of pixels
+	if (mx == -1) mx = 0;
+	if (my == -1) my = 0;
+
+	// multiply coords by video upscaling factors
+	mouse.x = (int32_t)floor(mx * video.dMouseXMul);
+	mouse.y = (int32_t)floor(my * video.dMouseYMul);
 
 	if (config.specialFlags2 & HARDWARE_MOUSE)
 	{
